@@ -8,11 +8,13 @@ use Carp 'croak';
 
 use base 'MIME::Entity';
 
-our $VERSION = '0.04';
+our $VERSION = '0.07';
 
 my $TEST = Test::Builder->new();
 
 my $DEBUG = 0;
+# for quietly failing .t tests which we expect to fail
+$Test::Email::QUIET_DIAG = 0;
 
 sub ok {
     my ($self, $test_href, $desc) = @_;
@@ -96,6 +98,22 @@ sub body_is {
     return $pass;
 }
 
+sub parts_ok {
+    my ($self, $part_count, $description) = @_;
+
+    my $pass = $TEST->is_num($part_count, scalar($self->parts()), $description);
+
+    return $pass;
+}
+
+sub mime_type_ok {
+    my ($self, $type, $description) = @_;
+
+    my $pass = $TEST->is_eq($type, $self->mime_type(), $description);
+
+    return $pass;
+}
+
 # run all tests against this email, return success
 sub _run_tests {
     my ($self, $test_href) = @_;
@@ -111,8 +129,8 @@ sub _run_tests {
 }
 
 my %test_for = (
-    header	=>	\&_test_header,
-    body	=>	\&_test_body,
+    header  =>  \&_test_header,
+    body    =>  \&_test_body,
 );
 
 # perform one test against one email
@@ -122,10 +140,10 @@ sub _test {
     _debug("in _test($self, $key, $test)");
 
     if (my $test_cref = $test_for{$key}) {
-	return $test_cref->($self, $test);
+        return $test_cref->($self, $test);
     }
     else {
-	return $test_for{header}->($self, $key, $test);
+        return $test_for{header}->($self, $key, $test);
     }
 }
 
@@ -137,7 +155,7 @@ sub _test_header {
     my $value = $self->head()->get($header) || '';
     chomp($value);
 
-    return _do_test($value, $test);
+    return _do_test($value, $test, $header);
 }
 
 sub _test_body {
@@ -146,25 +164,38 @@ sub _test_body {
     _debug("in _test_body($self, $test)");
 
     my $body = join '', @{ $self->body() };
-    return _do_test($body, $test);
+    return _do_test($body, $test, 'body');
 }
 
 sub _do_test {
-    my ($thing, $test) = @_;
+    my ($thing, $test, $what) = @_;
 
     _debug("Testing '$thing' against $test");
 
     my $type = ref $test;
     if ($type eq 'Regexp') {
-	return $thing =~ $test;
+        my $ret = $thing =~ $test;
+        if (!$ret && !$Test::Email::QUIET_DIAG) {
+            $TEST->diag("Email $what:");
+            $TEST->diag(sprintf <<DIAGNOSTIC, $thing, "doesn't match", $test);
+                  %s
+    %13s '%s'
+DIAGNOSTIC
+        }
+        return $ret;
     }
     elsif ($type eq '') {
         $thing =~ s/\n+$//;
         $test  =~ s/\n+$//;
-	return $thing eq $test;
+        my $ret = $thing eq $test;
+        if (!$ret && !$Test::Email::QUIET_DIAG) {
+            $TEST->diag("Email $what:");
+            $TEST->_is_diag($thing, 'eq', $test);
+        }
+        return $ret;
     }
     else {
-	croak "I don't know how to test for this type: '$type'";
+        croak "I don't know how to test for this type: '$type'";
     }
 }
 
@@ -206,6 +237,13 @@ Test::Email - Test Email Contents
   $email->body_ok($header_name, $value, "$header_name matches");
   $email->body_like($header_name, qr/regex/, "$header_name matches");
 
+  # how many MIME parts does the messages contain?
+  $email->parts_ok($parts_count, "there were $parts_count parts found");
+
+  # what is the MIME type of the firs part
+  my @parts = $email->parts(); # see MIME::Entity
+  $parts[0]->mime_type_ok('test/html', 'the first part is type text/html');
+
 =head1 DESCRIPTION
 
 Please note that this is ALPHA CODE. As such, the interface is likely to
@@ -221,8 +259,6 @@ the mailing process, which could cause unnecessary confusion.
 This module should be 100% self-explanatory. If not, then please look at
 L<Test::Simple> and L<Test::More> for clarification.
 
-This module does not yet test MIME, but it will soon.
-
 =head1 METHODS
 
 =over
@@ -249,6 +285,15 @@ Please note that tests for equality remove newlines from their operands
 before testing. This is because some email messages have newlines appended
 to them during mailing.
 
+=item C<my $ok = $email->parts_ok($parts_count, $description);>
+
+Check to see how many MIME parts this email contains. Each part is also a
+Test::Email object.
+
+=item C<my $ok = $email->mime_type_ok($expected_mime_type, $description);>
+
+Check the MIME type of an email or an email part.
+
 =back
 
 =head1 EXPORT
@@ -261,7 +306,7 @@ L<Test::Builder>, L<Test::Simple>, L<Test::More>, L<Test::POP3>
 
 =head1 TODO
 
-Much. This is only a first step.
+I am open to suggestions.
 
 =head1 AUTHOR
 
@@ -269,7 +314,7 @@ James Tolley, E<lt>james@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007 by James Tolley
+Copyright (C) 2007-2008 by James Tolley
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
